savedcmd_arch/arm64/include/generated/asm/kernel-hwcap.h := mkdir -p arch/arm64/include/generated/asm/; /bin/sh -e arch/arm64/tools/gen-kernel-hwcaps.sh arch/arm64/include/uapi/asm/hwcap.h > arch/arm64/include/generated/asm/kernel-hwcap.h
